@@ -1,6 +1,9 @@
-'''
-Copyright 2020 The Microsoft DeepSpeed Team
-'''
+# Copyright (c) Microsoft Corporation.
+# SPDX-License-Identifier: Apache-2.0
+
+# DeepSpeed Team
+
+import os
 from datetime import timedelta
 
 #############################################
@@ -13,7 +16,10 @@ TORCH_DISTRIBUTED_DEFAULT_PORT = 29500
 # (only if NCCL_BLOCKING_WAIT or NCCL_ASYNC_ERROR_HANDLING is set to 1).
 # To make an attempt at backwards compatibility with THD, we use an
 # extraordinarily high default timeout, given that THD did not have timeouts.
-default_pg_timeout = timedelta(minutes=30)
-
+default_pg_timeout = timedelta(minutes=int(os.getenv("DEEPSPEED_TIMEOUT", default=30)))
 INFERENCE_GENERIC_MODE = 'generic'
 INFERENCE_SPECIALIZED_MODE = 'specialized'
+
+CROSS_RANK = "CROSS_RANK"
+CROSS_SIZE = "CROSS_SIZE"
+LOCAL_RANK = 'LOCAL_RANK'
