@@ -8,6 +8,8 @@ from .partition_parameters import ZeroParamStatus
 from .partition_parameters import Init
 from .partition_parameters import GatheredParameters
 from .partition_parameters import register_external_parameter
+from .parameter_offload import DeepSpeedZeRoOffload
+from .partition_parameters import DeepSpeedTensorOverride
 
 from .tiling import TiledLinear
 from .tiling import TiledLinearReturnBias
