@@ -33,8 +33,12 @@ from deepspeed.accelerator import get_accelerator
 
 # Expert parallel group that the current rank belongs to.
 _EXPERT_PARALLEL_GROUP = {}
+# Mapping of expert parallel group to ranks
+_EXPERT_PARALLEL_GROUP_RANKS = {}
 # Expert data parallel group that the current rank belongs to.
 _EXPERT_DATA_PARALLEL_GROUP = {}
+# Mapping of expert data parallel group to ranks
+_EXPERT_DATA_PARALLEL_GROUP_RANKS = {}
 # dist world group needs to be cloned for some cases
 _WORLD_GROUP = None
 # ZeRO parameter  partitioning group that the current rank belongs to.
@@ -46,8 +50,6 @@ expert_tensor_parallel_world_size = 1
 # All to All quantized graident communication groups
 _ALL_TO_ALL_GROUP = {}
 
-_DATA_PARALLEL_GROUP = None
-
 mesh_device = None
 
 
@@ -64,6 +66,127 @@ def _ensure_divisibility(numerator, denominator):
     assert numerator % denominator == 0, '{} is not divisible by {}'.format(numerator, denominator)
 
 
+# ======== Start: Tensor Parallel Group Attributes ========
+
+# Intra-layer model parallel group that the current rank belongs to.
+_TENSOR_MODEL_PARALLEL_GROUP = None
+
+# Model parallel group (both intra- and pipeline) that the current rank belongs to.
+_MODEL_PARALLEL_GROUP = None
+# Data parallel group that the current rank belongs to.
+_DATA_PARALLEL_GROUP = None
+
+# These values enable us to change the mpu sizes on the fly.
+_MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE = None
+_MPU_TENSOR_MODEL_PARALLEL_RANK = None
+
+
+def _init_tp_mesh_device(tensor_model_parallel_size=1, data_parallel_size=None):
+    """Initialize model data parallel groups."""
+
+    global _DATA_PARALLEL_GROUP
+    global _MODEL_PARALLEL_GROUP
+    global _TENSOR_MODEL_PARALLEL_GROUP
+
+    if _TENSOR_MODEL_PARALLEL_GROUP is not None:
+        return
+
+    if data_parallel_size is None:
+        data_parallel_size = dist.get_world_size() // tensor_model_parallel_size
+
+    mesh_device = dist.initialize_mesh_device((data_parallel_size, tensor_model_parallel_size),
+                                              ("data_parallel", "tensor_parallel"))
+    _TENSOR_MODEL_PARALLEL_GROUP = mesh_device.get_group(mesh_dim="tensor_parallel")
+    _DATA_PARALLEL_GROUP = mesh_device.get_group(mesh_dim="data_parallel")
+
+    # They are always equal only in 2D (DP + TP) parallelism.
+    # _MODEL_PARALLEL_GROUP is assigned the same value as _TENSOR_MODEL_PARALLEL_GROUP
+    # to allow for future potential changes.
+    _MODEL_PARALLEL_GROUP = _TENSOR_MODEL_PARALLEL_GROUP
+
+    return _DATA_PARALLEL_GROUP, _MODEL_PARALLEL_GROUP
+
+
+def get_tensor_model_parallel_group():
+    """Get the tensor model parallel group the caller rank belongs to."""
+
+    assert _TENSOR_MODEL_PARALLEL_GROUP is not None, \
+        'intra_layer_model parallel group is not initialized'
+    return _TENSOR_MODEL_PARALLEL_GROUP
+
+
+def get_model_parallel_group():
+    """Get the model parallel group the caller rank belongs to."""
+
+    assert _MODEL_PARALLEL_GROUP is not None, \
+        'model parallel group is not initialized'
+    return _MODEL_PARALLEL_GROUP
+
+
+def get_data_parallel_group():
+    """Get the data parallel group the caller rank belongs to."""
+    assert _DATA_PARALLEL_GROUP is not None, \
+        'data parallel group is not initialized'
+    return _DATA_PARALLEL_GROUP
+
+
+def set_tensor_model_parallel_world_size(world_size):
+    """Set the tensor model parallel size"""
+    global _MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE
+    _MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE = world_size
+
+
+def get_tensor_model_parallel_world_size():
+    """Return world size for the tensor model parallel group."""
+    global _MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE
+    if _MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE is not None:
+        return _MPU_TENSOR_MODEL_PARALLEL_WORLD_SIZE
+    return dist.get_world_size(group=get_tensor_model_parallel_group())
+
+
+def get_model_parallel_world_size():
+    return get_tensor_model_parallel_world_size()
+
+
+def set_tensor_model_parallel_rank(rank):
+    """Set tensor model parallel rank."""
+    global _MPU_TENSOR_MODEL_PARALLEL_RANK
+    _MPU_TENSOR_MODEL_PARALLEL_RANK = rank
+
+
+def get_tensor_model_parallel_rank():
+    """Return my rank for the tensor model parallel group."""
+    global _MPU_TENSOR_MODEL_PARALLEL_RANK
+    if _MPU_TENSOR_MODEL_PARALLEL_RANK is not None:
+        return _MPU_TENSOR_MODEL_PARALLEL_RANK
+    return dist.get_rank(group=get_tensor_model_parallel_group())
+
+
+def get_model_parallel_rank():
+    return get_tensor_model_parallel_rank()
+
+
+def get_tensor_model_parallel_src_rank():
+    """Calculate the global rank corresponding to the first local rank
+    in the tensor model parallel group."""
+    global_rank = dist.get_rank()
+    local_world_size = get_tensor_model_parallel_world_size()
+    return (global_rank // local_world_size) * local_world_size
+
+
+def get_data_parallel_world_size():
+    """Return world size for the data parallel group."""
+    return dist.get_world_size(group=get_data_parallel_group())
+
+
+def get_data_parallel_rank():
+    """Return my rank for the data parallel group."""
+    return dist.get_rank(group=get_data_parallel_group())
+
+
+# ======== End: Tensor Parallel Group Attributes ========
+
+
 # Not currently used. Helper function to create a model (tensor) parallel group.
 def _create_model_parallel(model_parallel_size_):
     """
@@ -142,6 +265,7 @@ def _create_expert_and_data_parallel(expert_parallel_size_, use_data_before_expe
 
     # Build the expert data parallel groups.
     global _EXPERT_DATA_PARALLEL_GROUP
+    global _EXPERT_DATA_PARALLEL_GROUP_RANKS
 
     ep_stride = pp_stride // expert_parallel_size_
 
@@ -154,14 +278,15 @@ def _create_expert_and_data_parallel(expert_parallel_size_, use_data_before_expe
                 else:
                     ranks = range(pp_stage_start + i, pp_stage_start + pp_stride, expert_parallel_size_)
                 group = dist.new_group(ranks)
-                log_dist(
-                    f'Creating expert data parallel process group named {group_name} '
-                    f'with ranks: {list(ranks)}', [0])
+                log_dist(f'Creating expert data parallel process group named {group_name} with ranks: {list(ranks)}',
+                         [0])
                 if rank in ranks:
                     _EXPERT_DATA_PARALLEL_GROUP[group_name] = group
+                    _EXPERT_DATA_PARALLEL_GROUP_RANKS[group_name] = ranks
 
     # Build the expert parallel groups.
     global _EXPERT_PARALLEL_GROUP
+    global _EXPERT_PARALLEL_GROUP_RANKS
 
     # Only create group if it does not already exist
     if group_name not in _EXPERT_PARALLEL_GROUP:
@@ -175,6 +300,7 @@ def _create_expert_and_data_parallel(expert_parallel_size_, use_data_before_expe
                         f'with ranks: {list(ranks)}', [0])
                     if rank in ranks:
                         _EXPERT_PARALLEL_GROUP[group_name] = group
+                        _EXPERT_PARALLEL_GROUP_RANKS[group_name] = ranks
         else:
             for i in range(world_size // expert_parallel_size_):
                 ranks = range(i * expert_parallel_size_, (i + 1) * expert_parallel_size_)
@@ -183,6 +309,7 @@ def _create_expert_and_data_parallel(expert_parallel_size_, use_data_before_expe
                          f'with ranks: {list(ranks)}', [0])
                 if rank in ranks:
                     _EXPERT_PARALLEL_GROUP[group_name] = group
+                    _EXPERT_PARALLEL_GROUP_RANKS[group_name] = ranks
 
 
 def _get_expert_parallel_ranks(world_size,
@@ -289,6 +416,7 @@ def _create_expert_data_and_model_parallel(expert_parallel_size_, mpu, use_data_
         f"world size {world_size}, dp world size {dp_world_size}", [0])
 
     global _EXPERT_PARALLEL_GROUP, _EXPERT_DATA_PARALLEL_GROUP
+    global _EXPERT_PARALLEL_GROUP_RANKS, _EXPERT_DATA_PARALLEL_GROUP_RANKS
 
     group_name = f"ep_size_{expert_parallel_size_}"
 
@@ -301,11 +429,13 @@ def _create_expert_data_and_model_parallel(expert_parallel_size_, mpu, use_data_
             group = dist.new_group(ranks)
             if rank in list(ranks):
                 _EXPERT_PARALLEL_GROUP[group_name] = group
+                _EXPERT_PARALLEL_GROUP_RANKS[group_name] = ranks
 
         for ranks in expert_data_parallel_groups:
             group = dist.new_group(ranks)
             if rank in list(ranks):
                 _EXPERT_DATA_PARALLEL_GROUP[group_name] = group
+                _EXPERT_DATA_PARALLEL_GROUP_RANKS[group_name] = ranks
 
 
 def _get_max_expert_size():
@@ -336,6 +466,13 @@ def _get_expert_parallel_group(group_name):
     return _EXPERT_PARALLEL_GROUP[group_name]
 
 
+def _get_expert_parallel_group_ranks(group_name):
+    """Get the ranks of the expert parallel group the caller rank belongs to."""
+    assert group_name in _EXPERT_PARALLEL_GROUP_RANKS, \
+        'expert parallel group is not initialized'
+    return _EXPERT_PARALLEL_GROUP_RANKS[group_name]
+
+
 def _get_expert_parallel_group_dict():
     """Get the expert parallel group dict."""
     return _EXPERT_PARALLEL_GROUP
@@ -348,6 +485,13 @@ def _get_expert_data_parallel_group(group_name):
     return _EXPERT_DATA_PARALLEL_GROUP[group_name]
 
 
+def _get_expert_data_parallel_group_ranks(group_name):
+    """Get the ranks of the expert data parallel group the caller rank belongs to."""
+    assert group_name in _EXPERT_DATA_PARALLEL_GROUP_RANKS, \
+        'expert data parallel group is not initialized'
+    return _EXPERT_DATA_PARALLEL_GROUP_RANKS[group_name]
+
+
 def _get_expert_data_parallel_group_dict():
     """Get the expert data parallel group dict."""
     return _EXPERT_DATA_PARALLEL_GROUP
@@ -404,12 +548,26 @@ def _get_data_parallel_group():
     if mesh_device is not None:
         return mesh_device.get_group(mesh_dim="data_parallel")
     if mpu is not None:
-        return mpu.get_data_parallel_group()
+        if hasattr(mpu, 'initialize_sequence_parallel'):
+            return None
+        else:
+            return mpu.get_data_parallel_group()
 
     # Return the clone of dist world group
     return _clone_world_group()
 
 
+def _get_data_parallel_group_ranks():
+    """Get the ranks of data parallel group the caller rank belongs to."""
+    assert dist.is_initialized(), \
+        'dist is not initialized'
+    global mpu
+    if mpu is not None:
+        return mpu.get_data_parallel_group_ranks()
+    # Return all ranks
+    return range(dist.get_world_size())
+
+
 def _get_broadcast_src_rank():
     return dist.get_global_rank(_get_sequence_data_parallel_group(), 0)
 
@@ -452,16 +610,19 @@ def _get_data_parallel_world_size():
         return dist.get_world_size(mesh_device.get_group(mesh_dim="data_parallel"))
     global mpu
     if mpu is not None:
-        return mpu.get_data_parallel_world_size()
+        if hasattr(mpu, 'initialize_sequence_parallel'):
+            return None
+        else:
+            return mpu.get_data_parallel_world_size()
     return dist.get_world_size(group=_get_data_parallel_group())
 
 
 def _get_model_parallel_world_size():
     """Return world size for the model parallel group."""
     global mpu
-    if mpu is not None:
-        return mpu.get_model_parallel_world_size()
-    return 1
+    if mpu is None or hasattr(mpu, 'initialize_sequence_parallel'):
+        return 1
+    return mpu.get_model_parallel_world_size()
 
 
 def _get_data_parallel_rank():
@@ -470,7 +631,7 @@ def _get_data_parallel_rank():
 
 
 def _get_sequence_parallel_world_size():
-    """Return world size for the model parallel group."""
+    """Return world size for the sequence parallel group."""
     global mpu
     if mesh_device is not None:
         return dist.get_world_size(mesh_device.get_group(mesh_dim="sequence_parallel"))
@@ -480,7 +641,7 @@ def _get_sequence_parallel_world_size():
 
 
 def _get_sequence_parallel_rank():
-    """Return my rank for the data parallel group."""
+    """Return my rank for the sequence parallel group."""
     global mpu
     if mpu is not None and hasattr(mpu, 'get_sequence_parallel_rank'):
         return mpu.get_sequence_parallel_rank()
