@@ -32,9 +32,6 @@ class CPUOpBuilder(OpBuilder):
 
     def cxx_args(self):
         args = ['-O3', '-g', '-Wno-reorder']
-        CPU_ARCH = self.cpu_arch()
-        SIMD_WIDTH = self.simd_width()
-        args += [CPU_ARCH, '-fopenmp', SIMD_WIDTH]
         return args
 
     def libraries_args(self):
